#include <Arduino.h>
#include <ESPMegaPRO_OOP.hpp>

#define DISPLAY_FETCH_TIMEOUT 100 // ms

class ESPMegaDisplay
{
    public:
        ESPMegaDisplay(HardwareSerial *displayAdapter);
        void begin();
        void loop();
        void reset();
        void setBrightness(int value);
        void setVolume(int value);
        void jumpToPage(int page);
        void setString(char* component, char* value);
        void setNumber(char* component, int value);
        char* getString(char* component);
        bool getStringToBuffer(char* component, char* buffer, uint8_t buffer_size);
        uint32_t getNumber(char* component);
        void handlePwmStateChange(uint8_t pin, uint16_t value);
        void handleInputStateChange(uint8_t pin, bool state);
        void registerPushCallback(std::function<void(uint8_t, uint8_t)> callback);
        void registerPopCallback(std::function<void(uint8_t, uint8_t)> callback);
        void registerPageChangeCallback(std::function<void(uint8_t)> callback);
        void unregisterPushCallback();
        void unregisterPopCallback();
        void unregisterPageChangeCallback();
    protected:
        uint8_t currentPage;
        uint8_t rx_buffer_index;
        char rx_buffer[256];
        char tx_buffer[256];
        bool recieveSerialCommand();
        bool recieveSerialCommand(bool process);
        void processSerialCommand();
        void processTouchPayload();
        void processPageReportPayload();
        void sendStopBytes();
        void sendCommand(char* command);
        bool payloadIsValid();
        bool waitForValidPayload(uint32_t timeout);
        HardwareSerial *displayAdapter;
        std::function<void(uint8_t, uint8_t)> pushCallback;
        std::function<void(uint8_t, uint8_t)> popCallback;
        std::function<void(uint8_t)> pageChangeCallback;
};