#pragma once
#include <ExpansionCard.hpp>
#include <DigitalInputCard.hpp>
#include <DigitalOutputCard.hpp>
#include <ESPMegaIoT.hpp>
#include <Arduino.h>
#include <Wire.h>
#include <FRAM.h>
#include <TimeLib.h>
#include <DS1307RTC.h>
#include <time.h>

struct rtctime_t {
    uint8_t hours;
    uint8_t minutes;
    uint8_t seconds;
    uint8_t day;
    uint8_t month;
    uint16_t year;
};

#define FRAM_ADDRESS 0x56
#define INPUT_BANK_A_ADDRESS 0x21
#define INPUT_BANK_B_ADDRESS 0x22
#define PWM_BANK_ADDRESS 0x5F
#define RTC_ADDRESS 0x68

class ESPMegaPRO {
    public:
        ESPMegaPRO();
        bool begin();
        void loop();  
        bool installCard(uint8_t slot, ExpansionCard* card);
        bool updateTimeFromNTP();
        void enableIotModule();
        rtctime_t getTime();
        void setTime(int hours, int minutes, int seconds, int day, int month, int year);
        FRAM fram;
        DigitalInputCard inputs = DigitalInputCard(INPUT_BANK_A_ADDRESS, INPUT_BANK_B_ADDRESS);
        DigitalOutputCard outputs = DigitalOutputCard(PWM_BANK_ADDRESS);
        ESPMegaIoT iot = ESPMegaIoT();
    private:
        ExpansionCard* cards[255];
        bool cardInstalled[255];
        uint8_t cardCount = 0;
};